###################################################################################################
# zmsdocument.py
#
# Implementation of class ZMSDocument (see below).
#
# This program is free software; you can redistribute it and/or
# modify it under the terms of the GNU General Public License
# as published by the Free Software Foundation; either version 2
# of the License, or (at your option) any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License
# along with this program; if not, write to the Free Software
# Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
###################################################################################################

# Imports.
from Globals import HTMLFile, MessageDialog, Persistent
import string
import urllib
# Product Imports.
from zmscontainerobject import ZMSContainerObject
import _globals
import _printablemanager


###################################################################################################
###################################################################################################
###   
###   C o n s t r u c t o r ( s )
###   
###################################################################################################
###################################################################################################

manage_addZMSDocumentForm = HTMLFile('manage_addzmsdocumentform', globals()) 
def manage_addZMSDocument(self, lang, manage_lang, _sort_id, REQUEST):
  """ manage_addZMSDocument """
    
  if REQUEST['btn'] == self.getLangStr('BTN_INSERT',manage_lang):
    
    ##### Create ####
    obj = ZMSDocument(self.getNewId(),_sort_id+1)
    self._setObject(obj.id, obj)
        
    for oItem in self.objectValues():
      if oItem.id == obj.id:
        ##### Object State ####
        oItem.setObjStateNew(REQUEST)
        ##### Init Properties ####
        oItem.manage_changeProperties(lang,manage_lang,REQUEST)
        ##### VersionManager ####
        oItem.__autoCommit__(REQUEST)
                
    ##### Normalize Sort-IDs ####
    self.normalizeSortIds()
        
    # Return with message.
    message = self.getLangStr('MSG_INSERTED',manage_lang)%obj.display_type(REQUEST)
    if REQUEST.RESPONSE:
      REQUEST.RESPONSE.redirect('%s/%s/manage_main?lang=%s&manage_lang=%s&manage_tabs_message=%s'%(self.absolute_url(),obj.id,lang,manage_lang,urllib.quote(message)))
          
  else:         
    if REQUEST.RESPONSE:
      REQUEST.RESPONSE.redirect('%s/manage_main?lang=%s&manage_lang=%s'%(self.absolute_url(),lang,manage_lang))


###################################################################################################
###################################################################################################
###
###   C l a s s
###
###################################################################################################
###################################################################################################
class ZMSDocument(
	ZMSContainerObject,
	_printablemanager.PrintableManager	# Printable manager.
	):

    # Properties.
    # -----------
    meta_type = "ZMSDocument"
    icon = "misc_/zms/zmsdocument.gif"
    icon_disabled = "misc_/zms/zmsdocument_disabled.gif"
    
    # Management Options.
    # -------------------
    manage_options = ( 
	{'label': 'TAB_EDIT',         'action': 'manage_main'},
	{'label': 'TAB_PROPERTIES',   'action': 'manage_properties'},
	{'label': 'TAB_ACCESS',       'action': 'manage_users'},
	{'label': 'TAB_IMPORTEXPORT', 'action': 'manage_importexport'},
	{'label': 'TAB_TASKS',        'action': 'manage_tasks'},
	{'label': 'TAB_PREVIEW',      'action': 'preview_html'}, # empty string defaults to index_html
	) 

    # Management Permissions.
    # -----------------------
    __authorPermissions__ = (
		'manage','manage_main',
		'manage_addZMSTextarea','manage_addZMSGraphic','manage_addZMSTable','manage_addZMSFile','manage_addZMSTeaserContainer','manage_addZMSNote','manage_addZMSLinkContainer',
		'manage_deleteObjs','manage_moveObjUp','manage_moveObjDown','manage_cutObjects','manage_copyObjects','manage_pasteObjs',
		'manage_properties','manage_changeProperties',
		'manage_sitemap','manage_search','manage_tasks',
		'manage_rollbackVersionChanges','manage_requestVersionChanges',
		'manage_users',
		)
    __editorPermissions__ = (
		'manage_commitVersionChanges',
		)
    __administratorPermissions__ = (
		)
    __ac_permissions__=(
		('ZMS Author', __authorPermissions__),
		('ZMS Editor', __editorPermissions__),
		('ZMS Administrator', __administratorPermissions__),
		('Import/Export objects', ('manage_importexport','manage_import','manage_export',) ),
		# ('Manage users', ('manage_users',) ),
		)

    # Properties.
    # -----------
    __obj_attrs__ = {
        # Changed by
        'change_uid':{'datatype':'LangString','lang_inherit':_globals.LI_FORBIDDEN},
        'change_dt':{'datatype':'LangDate','lang_inherit':_globals.LI_FORBIDDEN},
        # Versioned by
        'work_uid':{'datatype':'LangString','lang_inherit':_globals.LI_FORBIDDEN},
        'work_dt':{'datatype':'LangDate','lang_inherit':_globals.LI_FORBIDDEN},
        # Properties
        'sort_id':{'datatype':'string','native':1},
        'active':{'datatype':'LangInt','type':'checkbox','lang_inherit':_globals.LI_OBLIGATE},
        'attr_active_start':{'datatype':'LangDate','lang_inherit':_globals.LI_OBLIGATE},
        'attr_active_end':{'datatype':'LangDate','lang_inherit':_globals.LI_OBLIGATE},
        'title':{'datatype':'LangString','size':40},
        'titleshort':{'datatype':'LangString','size':20},
        'titleimage':{'datatype':'LangMyImage'},
        'attr_alias':{'datatype':'int'},
        'attr_cacheable':{'datatype':'boolean','default':1},
        # Meta-Data
        'attr_dc_coverage':{'datatype':'string','default':'obligation'},
        # Meta-Dictionaries        
        '$metadict':{'datatype':'MetaDict'},
    }

###################################################################################################
